import json
import subprocess
import os.path
from os import path


class ExifHandler:

    def __init__(self, target):
        self.target = target
        self.tags = [
            'FileDescription',
            'FileVersion',
            'InternalName',
            'LegalCopyright',
            'OriginalFileName',
            'ProductName',
            'ProductVersion',
            'Comments']
        self.fileMeta = {}

    def run(self):
        if(path.exists(self.target)):
            cmd = "exiftool -json " + self.target
            process = subprocess.Popen(
                cmd,
                shell=True,
                stdout=subprocess.PIPE,
                stderr=subprocess.PIPE)
            (result, error) = process.communicate()
            rc = process.wait()
            process.stdout.close()
            if len(result) >= 2:
                jsonData = json.loads(result.decode('utf-8'))[0]
                for tag in self.tags:
                    try:
                        self.fileMeta[tag] = jsonData[tag]
                    except KeyError:
                        self.fileMeta[tag] = ''
        return self.fileMeta



import os
import mimetypes
import hashlib
import tarfile
import gzip
import zlib
import pprint
from zipfile import ZipFile
from . import File


class ArchiveHandler:

    def __init__(self, eH, filepath, filename, filetype, slugify, checksum):
        self.debug = eH
        self.filepath = filepath
        self.filename = filename
        self.filetype = filetype
        self.slugify = slugify
        self.checksum = checksum
        self.archive_handler(filepath, filename, filetype, slugify, checksum)

    def preload_handlers(self):
        return {
            'application/zip': self.unzip_archive,
            'application/java-archive': self.unzip_archive,
            'application/gzip': self.ungzip_archive,
            'application/zlib': self.unzlib_archive,
            'application/tar': self.untar_archive,
            'application/x-tar': self.untar_archive
        }

    def archive_handler(self, filepath, filename, filetype, slugify, checksum):
        # Pending recursive extraction
        try:
            handlers = self.preload_handlers()
            handler = handlers[filetype]
            source = os.path.join(filepath, filename)
            target = os.path.join(filepath, slugify)
            self.debug.info('handling ' + filetype)
            handler(filepath, filename, slugify)
        except KeyError:
            self.debug.error('handler not implemented for ' + filetype)

    def unzlib_archive(self, filepath, filename, slugify):
        f = open(os.path.join(filepath, filename), 'rb').read()
        df = os.path.splitext(filename)[0]+'.dmp'
        tf = os.path.join(filepath, df)
        f_out = open(tf, 'wb')
        f_out.write(zlib.decompress(f))
        f_out.close()

    def ungzip_archive(self, filepath, filename, slugify):
        if filename.endswith("tgz"):
            self.untar_archive(filepath, filename, slugify)
        elif filename.endswith("tar.gz"):
            self.untar_archive(filepath, filename, slugify)
        else:
            print('gzip')
            f = gzip.open(os.path.join(filepath, filename), 'r')
            file_content = f.read()
            file_content = file_content.decode('utf-8')
            nm = os.path.splitext(filename)[0]
            tf = os.path.join(filepath, nm)
            f_out = open(tf, 'w+')
            f_out.write(file_content)
            f.close()
            f_out.close()

    def unzip_archive(self, filepath, filename, slugify):
        with ZipFile(os.path.join(filepath, filename), 'r') as zipObj:
            zipObj.extractall(os.path.join(filepath, slugify))

    def untar_archive(self, filepath, filename, slugify):
        if filename.endswith("tar.gz"):
            TarFile = tarfile.open(os.path.join(filepath, filename), "r:gz")
            TarFile.extractall(os.path.join(filepath, slugify))
            TarFile.close()
        elif filename.endswith("tgz"):
            TarFile = tarfile.open(os.path.join(filepath, filename), "r:gz")
            TarFile.extractall(os.path.join(filepath, slugify))
            TarFile.close()
        elif filename.endswith("tar"):
            TarFile = tarfile.open(os.path.join(filepath, filename), "r:")
            TarFile.extractall(os.path.join(filepath, slugify))
            TarFile.close()



import os
import time
import mimetypes
import hashlib
import tarfile
import gzip
import csv
import sys
from slugify import slugify
from zipfile import ZipFile
from tabulate import tabulate
from . import File
from . import ArchiveHandler
from . import ErrorHandler
from . import FolderScanner


class FileScanner:

    def __init__(self, filepath):

        self.debug = ErrorHandler.ErrorHandler(__name__)
        ts = time.time()

        # Files found during scanner
        self.filelist = []

        # Files to Exclude from ArchiveHandler
        self.exfiles = {}

        # List of folders
        self.folders = {}
        self.symbols = {}

        self.rootfolder = os.path.dirname(filepath)
        if not self.rootfolder:
            self.rootfolder = '.'
        self.scanFile(filepath, '-')
        self.rfname = slugify(filepath)

        self.fresults = []
        self.aresults = []
        self.bsaqueue = []
        self.mresults = []

        self.lcounter = 0

        for fO in self.filelist:
            row = [
                fO.checksum,
                fO.filename,
                fO.filetype,
                fO.isbinary,
                fO.isarchive,
                fO.forceign,
                fO.plang,
                fO.pfile,
                fO.filepath,
                fO.symstat]
            if fO.metadata:
                mrow = [
                    fO.metadata['OriginalFileName'],
                    fO.metadata['FileVersion'],
                    fO.metadata['InternalName'],
                    fO.metadata['LegalCopyright'],
                    fO.metadata['ProductName'],
                    fO.metadata['ProductVersion'],
                    fO.metadata['Comments']]
                if fO.metadata['OriginalFileName']:
                    self.mresults.append(mrow)
            if fO.isbinary and not fO.isarchive and not fO.forceign:
                self.bsaqueue.append(row)
            elif fO.isarchive and not fO.forceign:
                self.aresults.append(row)
            else:
                if not fO.forceign:
                    self.fresults.append(row)

        headers = [
            "Checksum",
            "Filename",
            "MimeType",
            "Binary",
            "Archive",
            "Ignore",
            "Language",
            "Parent",
            "Filepath",
            "Symbols"]

        with open(self.rfname+'-archive.csv', 'w', newline='') as file:
            writer = csv.writer(file)
            writer.writerow(headers)
            for row in self.aresults:
                writer.writerow(row)

        with open(self.rfname+'-files.csv', 'w', newline='') as file:
            writer = csv.writer(file)
            writer.writerow(headers)
            for row in self.fresults:
                writer.writerow(row)
            for row in self.bsaqueue:
                writer.writerow(row)

        headers = [
            "OriginalFileName",
            "FileVersion",
            "InternalName",
            "LegalCopyright",
            "ProductName",
            "ProductVersion",
            "Comments"]

        with open(self.rfname+'-metadata.csv', 'w', newline='') as file:
            writer = csv.writer(file)
            writer.writerow(headers)
            for row in self.mresults:
                writer.writerow(row)

        headers = [
            "Checksum",
            "Symbols"]

        rowsym = []
        for key in self.symbols:
            rowsym.append([key, self.symbols[key]])

        with open(self.rfname+'-symbols.csv', 'w', newline='') as file:
            writer = csv.writer(file)
            writer.writerow(headers)
            for row in rowsym:
                writer.writerow(row)

        # print(self.symbols)
        # for key in self.symbols:
        #    print(key, '->', self.symbols[key])

        zipObj = ZipFile(self.rfname+'-reports.zip', 'w')
        zipObj.write(self.rfname+'-archive.csv')
        zipObj.write(self.rfname+'-files.csv')
        zipObj.write(self.rfname+'-metadata.csv')
        zipObj.write(self.rfname+'-symbols.csv')
        zipObj.close()
        os.remove(self.rfname+'-archive.csv')
        os.remove(self.rfname+'-files.csv')
        os.remove(self.rfname+'-metadata.csv')
        os.remove(self.rfname+'-symbols.csv')

        dt = str(round(time.time() - ts, 2))
        self.debug.info('Scan took ' + dt + ' seconds')

    def addQuotes(self, string):
        if string.startswith("'") and string.endswith("'"):
            return string
        elif string.startswith('"') and string.endswith('"'):
            return string
        else:
            return '"'+string+'"'

    def progress(self, count, total):
        bar_len = 60
        filled_len = int(round(bar_len * count / float(total)))

        percents = round(100.0 * count / float(total), 1)
        bar = '=' * filled_len + '-' * (bar_len - filled_len)

        sys.stdout.write(
            '[%s] %s%s %s\r' % (bar, percents, '%', 'scanning...'))
        sys.stdout.flush()

    def scanFolder(self, rootfolder, pfile, lock):
        if lock == 0:
            eMSG = "* Preparing files to be scanned, please wait..."
            print(eMSG)
            kount = [len(files) for _, _, files in os.walk(rootfolder)]
            self.file_count = kount
            self.file_count = sum(self.file_count)
            print(' ', '>', self.file_count, 'files added to the queue.')
            self.lcounter = 0

        for (dirpath, dirnames, filenames) in os.walk(rootfolder):
            for f in filenames:
                if lock == 0:
                    self.lcounter += 1
                    self.progress(self.lcounter, self.file_count)
                fileInfo = File.File(self.debug, dirpath, f, pfile)
                symbols = fileInfo.get_symbols()
                self.filelist.append(fileInfo)

                if len(symbols) >= 1:
                    tmpCSV = symbols[list(symbols.keys())[0]].split(',', 1)
                    self.symbols[tmpCSV[0]] = tmpCSV[1]
            for d in dirnames:
                self.folders[d] = dirpath
        if lock == 0:
            sys.stdout.flush()
            print('')
        for ar in self.filelist:
            if ar.isarchive:
                print(' ', '>', ar.filename)
                archfile = os.path.join(ar.filepath, ar.filename)
                if archfile not in self.exfiles.keys():
                    target = os.path.join(ar.filepath, ar.slugify)
                    self.exfiles[archfile] = target
                    ArchiveHandler.ArchiveHandler(
                        self.debug,
                        ar.filepath,
                        ar.filename,
                        ar.filetype,
                        ar.slugify,
                        ar.checksum
                    )
                    self.scanFolder(target, ar.checksum, 1)
        lock = 1

    def scanFile(self, filepath, pfile):
        if len(pfile.strip()) <= 1:
            eMSG = "* Scanning, please wait..."
            print(eMSG)

        basename = os.path.basename(filepath)
        fileInfo = File.File(self.debug, self.rootfolder, basename, pfile)
        self.filelist.append(fileInfo)
        symbols = fileInfo.get_symbols()
        if len(symbols) >= 1:
            tmpCSV = symbols[list(symbols.keys())[0]].split(',', 1)
            self.symbols[tmpCSV[0]] = tmpCSV[1]

        for ar in self.filelist:
            if ar.isarchive:
                print(' ', '>', ar.filename)
                archfile = os.path.join(ar.filepath, ar.filename)
                if archfile not in self.exfiles.keys():
                    target = os.path.join(ar.filepath, ar.slugify)
                    self.exfiles[archfile] = target
                    ArchiveHandler.ArchiveHandler(
                        self.debug,
                        ar.filepath,
                        ar.filename,
                        ar.filetype,
                        ar.slugify,
                        ar.checksum
                    )
                    self.scanFolder(target, ar.checksum, 0)



import os
import lief
import re
import mimetypes
import hashlib
import os.path
import subprocess
from os import path
from string import digits
from . import CtagsHandler


class FileHandler:

    def __init__(self, errorHandler, filepath, filename, filetype, checksum):
        self.debug = errorHandler
        self.filepath = filepath
        self.filename = filename
        self.filetype = filetype
        self.checksum = checksum

    def preload_handlers(self):
        # This should be replaced by a BD
        return {
            'text/x-c': self.handle_cplusplus,
            'text/x-c++': self.handle_cplusplus,
            'text/x-python': self.handle_python,
            'text/x-perl': self.handle_perl,
            'text/x-ruby': self.handle_ruby,
            'text/x-rust': self.handle_rust,
            'text/x-java': self.handle_java,
            'text/x-objective-c': self.handle_objectivec,
            'application/x-mach-binary': self.handle_mach_o,
            'application/x-sharedlib': self.handle_sharedlib,
            'application/x-dosexec': self.handle_strings,
            'application/x-archive': self.handle_ar,
            'font/sfnt': self.handle_strings,

            'application/octet-stream': self.ignore,
            'application/x-ms-pdb': self.ignore,
            'image/vnd.microsoft.icon': self.ignore,
            'text/x-shellscript': self.ignore,
            'text/xml': self.ignore,
            'application/csv': self.ignore,
            'text/x-tex': self.ignore,
            'text/x-makefile': self.ignore,
            'application/json': self.ignore,
            'text/html': self.ignore,
            'image/x-portable-pixmap': self.ignore,
            'image/webp': self.ignore,
            'image/png': self.ignore,
            'image/x-tga': self.ignore,
            'image/g3fax': self.ignore,
            'image/gif': self.ignore,
            'image/jpeg': self.ignore,
            'application/x-wine-extension-ini': self.ignore,
            'audio/mpeg': self.ignore,
            'audio/x-wav': self.ignore,
            'video/mp4': self.ignore,
            'inode/x-empty': self.ignore
        }

    def run_handler(self):
        try:
            handlers = self.preload_handlers()
            handler = handlers[self.filetype]
            # print(self.filetype, self.filename)
            return handler(self.filepath, self.filename, self.checksum)
        except KeyError:
            self.debug.error('handler not implemented for ' + self.filetype)
            fullPath = os.path.join(self.filepath, self.filename)
            self.debug.error('skipping ' + fullPath)

    def ignore(self, filepath, filename, checksum):
        self.debug.info('skipping ' + self.filename)
        return ''

    def handle_ar(self, filepath, filename, checksum):
        fullpath = filepath+"/"+filename
        if(path.exists(fullpath)):
            cmd = 'ar t ' + fullpath
            process = subprocess.Popen(
                cmd,
                shell=True,
                stdout=subprocess.PIPE,
                stderr=subprocess.PIPE)
            (result, error) = process.communicate()
            rc = process.wait()
            process.stdout.close()
            rstTXT = result.decode('utf-8')
            results = self.stripNonAlphaNum(' '.join(rstTXT.split()))
            prow = checksum + "," + ",".join(results)
            prow = prow + "," + os.path.splitext(filename)[0]

            cmd = 'strings -n 5 ' + fullpath
            process = subprocess.Popen(
                cmd,
                shell=True,
                stdout=subprocess.PIPE,
                stderr=subprocess.PIPE)
            (result, error) = process.communicate()
            rc = process.wait()
            process.stdout.close()
            rstTXT = result.decode('utf-8')
            row = ""
            results = self.stripNonAlphaNum(','.join(rstTXT.split()))
            for x in results:
                if x.startswith('_Z'):
                    row = row + ",".join(self.demangle(x))
            row = list(set(row.split(',')))

            prow = prow + ",".join(row) + "," + os.path.splitext(filename)[0]

            return prow

    def handle_strings(self, filepath, filename, checksum):
        fullpath = filepath+"/"+filename
        if(path.exists(fullpath)):
            cmd = 'strings -n 5 ' + fullpath
            process = subprocess.Popen(
                cmd,
                shell=True,
                stdout=subprocess.PIPE,
                stderr=subprocess.PIPE)
            (result, error) = process.communicate()
            rc = process.wait()
            process.stdout.close()
            rstTXT = result.decode('utf-8')
            results = self.stripNonAlphaNum(' '.join(rstTXT.split()))
            prow = checksum + ",".join(results)
            prow = prow + "," + os.path.splitext(filename)[0]
            return prow

    def handle_sharedlib(self, filepath, filename, checksum):
        fullPath = os.path.join(filepath, filename)
        libSO = lief.parse(fullPath)
        symbols = []
        iter = filter(lambda e: e.exported, libSO.dynamic_symbols)
        for idx, lsym in enumerate(iter):
            symbols.extend(self.demangle(lsym.name))
        symbols = list(set(symbols))
        prow = checksum + ','
        prow = prow + ",".join(symbols)
        prow = prow + "," + os.path.splitext(filename)[0]
        return prow

    def handle_mach_o(self, filepath, filename, checksum):
        fullPath = os.path.join(filepath, filename)
        symbols = []
        if os.stat(fullPath).st_size <= 1048576:
            libSO = lief.parse(fullPath)
            remove_digits = str.maketrans(',', ',', digits)
            for i in libSO.symbols:
                symbol = i.name
                symbol = re.sub("[^a-zA-Z0-9]+", ",", symbol)
                symbol = re.sub("\d+", ",", symbol)
                symbols.extend(symbol.split(','))
            symbols = list(set(symbols))
            while("" in symbols):
                symbols.remove("")
            prow = checksum + ','
            prow = prow + ",".join(symbols)

        if len(symbols) <= 1:
            cmd = 'strings -n 5 ' + fullPath
            process = subprocess.Popen(
                cmd,
                shell=True,
                stdout=subprocess.PIPE,
                stderr=subprocess.PIPE)
            (result, error) = process.communicate()
            rc = process.wait()
            process.stdout.close()
            rstTXT = result.decode('utf-8')
            row = ""
            results = self.stripNonAlphaNum(','.join(rstTXT.split()))
            for sym in results:
                if sym.startswith('_Z'):
                    row = row + ",".join(self.demangle(sym))
            row = list(set(row.split(',')))
            prow = checksum + ','
            prow = prow + ",".join(row)

        prow = prow + "," + os.path.splitext(filename)[0]
        return prow

    def handle_objectivec(self, filepath, filename, checksum):
        fname = os.path.splitext(filename)[0]
        target = os.path.join(filepath, filename)
        ctags = CtagsHandler.CtagsHandler(target)
        ctags.setLang('objectivec')
        ctags.setLangMap('objectivec:.h.m')
        rst = ','.join([checksum, ctags.run(), fname])
        return rst

    def handle_rust(self, filepath, filename, checksum):
        fname = os.path.splitext(filename)[0]
        target = os.path.join(filepath, filename)
        ctags = CtagsHandler.CtagsHandler(target)
        ctags.setLang('Rust')
        ctags.setLangMap('Rust:.rs')
        rst = ','.join([checksum, ctags.run(), fname])
        return rst

    def handle_ruby(self, filepath, filename, checksum):
        fname = os.path.splitext(filename)[0]
        target = os.path.join(filepath, filename)
        ctags = CtagsHandler.CtagsHandler(target)
        ctags.setLang('ruby')
        ctags.setLangMap('ruby:+.rake')
        rst = ','.join([checksum, ctags.run(), fname])
        return rst

    def handle_perl(self, filepath, filename, checksum):
        fname = os.path.splitext(filename)[0]
        target = os.path.join(filepath, filename)
        ctags = CtagsHandler.CtagsHandler(target)
        ctags.setLang('Perl')
        ctags.setLangMap('Perl:+.t')
        rst = ','.join([checksum, ctags.run(), fname])
        return rst

    def handle_cplusplus(self, filepath, filename, checksum):
        fname = os.path.splitext(filename)[0]
        target = os.path.join(filepath, filename)
        ctags = CtagsHandler.CtagsHandler(target)
        ctags.setOption('--kinds-C++=+l')
        ctags.setOption('-o -')
        rst = ','.join([checksum, ctags.run(), fname])
        return rst

    def handle_python(self, filepath, filename, checksum):
        fname = os.path.splitext(filename)[0]
        target = os.path.join(filepath, filename)
        ctags = CtagsHandler.CtagsHandler(target)
        ctags.setLang('python')
        ctags.setOption('--python-kinds=-iv')
        rst = ','.join([checksum, ctags.run(), fname])
        return rst

    def handle_java(self, filepath, filename, checksum):
        fname = os.path.splitext(filename)[0]
        target = os.path.join(filepath, filename)
        ctags = CtagsHandler.CtagsHandler(target)
        ctags.setLang('Java')
        ctags.setLangMap('java:+.aj')
        rst = ','.join([checksum, ctags.run(), fname])
        return rst

    def stripNonAlphaNum(self, text):
        return re.compile(r'\W+', re.UNICODE).split(text)

    def demangle(self, name):
        s = re.sub("[^a-zA-Z]+", ",", name)
        s.replace(',,', ',')
        lst = s.split(",")
        lst = (map(lambda x: x.lower(), lst))
        lst = set(lst)
        # dstring = demangle(name)
        # results = self.stripNonAlphaNum(dstring)
        # return ' '.join(results).split()
        return lst



class CmdFactory:

    def __init__(self, targetPath):
        self.targetPath = targetPath
        self.cmdOptions = ["ctags"]

    def setLang(self, langName):
        langStr = "-x -R --fields=+l"
        self.cmdOptions.append(langStr)
        langStr = "--languages="+langName
        self.cmdOptions.append(langStr)

    def setLangMap(self, langMap):
        langStr = "--langmap="+langMap
        self.cmdOptions.append(langStr)

    def setOption(self, cmdOpt):
        self.cmdOptions.append(cmdOpt)

    def buildCmd(self):
        cmdToStr = ' '.join([str(elem) for elem in self.cmdOptions])
        return cmdToStr + ' ' + self.targetPath



#!/usr/bin/env python3
import os
import sys
import click
import calendar
import time
import terminal_banner
from shutil import which
from pathlib import Path
from thundera.libs import ErrorHandler
from thundera.libs import FileScanner
from thundera.libs import FolderScanner
from thundera.libs import File


os.environ['LANG'] = 'C.UTF-8'
os.environ['LC_ALL'] = 'C.UTF-8'

debug = ErrorHandler.ErrorHandler(__name__)


@click.command()
@click.argument(
    'target',
    type=click.Path(exists=True)
)
@click.option(
    '-wd',
    '--workdir',
    default=str(calendar.timegm(time.gmtime())),
    help='Working directory'
)
def cli(target, workdir):

    """ Thundera BSA """
    banner_txt = "Thundera Binary Static Analysis (BSA)"
    banner_obj = terminal_banner.Banner(banner_txt)
    print(banner_obj)
    print('')

    cmdlist = ["ctags", "readelf", "exiftool", "strings"]
    for cmd in cmdlist:
        if which(cmd) is None:
            eMSG = "Thundera requires "+cmd+" to run"
            debug.error(eMSG)
            eMSG = "Please install to continue..."
            debug.error(eMSG)
            exit()

    if os.name == "posix":
        if target.endswith('/'):
            target = target[:-1]
        if os.path.isdir(target):
            eMSG = "Scanning the folder "+target
            print(eMSG)
            scanner = FolderScanner.FolderScanner(target)
        elif os.path.isfile(target):
            eMSG = "Scanning the file "+target
            print(eMSG)
            scanner = FileScanner.FileScanner(target)
        else:
            debug.info('*** Target Path: %s' % target)
            debug.info('*** Working Directory: %s' % workdir)
            eMSG = "The target " + target
            eMSG = eMSG + " it\'s a special file (socket, FIFO, device file)"
            debug.error(eMSG)
            print(eMSG)
            print('')
    else:
        eMSG = "Thundera BSA only supports POSIX based OS"
        debug.error(eMSG)
        print(eMSG)
        print('')


if __name__ == '__main__':
    cli()



import subprocess
from . import CmdFactory


class CtagsHandler:

    def __init__(self, target):
        self.target = target
        self.cmd = CmdFactory.CmdFactory(target)
        self.symlst = []

    def setLang(self, langName):
        self.cmd.setLang(langName)

    def setLangMap(self, langMap):
        self.cmd.setLangMap(langMap)

    def setOption(self, cmdOpt):
        self.cmd.setOption(cmdOpt)

    def run(self):
        cmd = self.cmd.buildCmd()
        process = subprocess.Popen(
            cmd,
            shell=True,
            stdout=subprocess.PIPE,
            stderr=subprocess.PIPE)
        (result, error) = process.communicate()
        rc = process.wait()
        process.stdout.close()
        for line in result.decode('utf-8').splitlines():
            cols = line.split()
            if len(cols) >= 2:
                if len(cols[0]) >= 5:
                    self.symlst.append(cols[0])
        self.symlst = sorted(set(self.symlst))
        return ','.join(self.symlst)



#!/usr/bin/python3
import pathlib
import sys
from setuptools import setup, find_packages

HERE = pathlib.Path(__file__).parent
VERSION = '0.1.16'
PACKAGE_NAME = 'thundera-bsa'
AUTHOR = 'Oscar Valenzuela B.'
AUTHOR_EMAIL = 'thunderabsa-pypi@amazon.com'
URL = 'https://github.com/amazon-ospo/thunderabsa-cli'
LICENSE = 'Apache-2.0'
DESCRIPTION = 'Command Line Interface for ThunderaBSA'
LONG_DESCRIPTION = (HERE / "README.md").read_text()
LONG_DESC_TYPE = "text/markdown"
INSTALL_REQUIRES = [
      'click',
      'terminal_banner',
      'python-magic',
      'python-slugify',
      'binaryornot',
      'tabulate',
      'numpy',
      'lief'
]

setup(
    name=PACKAGE_NAME,
    version=VERSION,
    description=DESCRIPTION,
    long_description=LONG_DESCRIPTION,
    long_description_content_type=LONG_DESC_TYPE,
    author=AUTHOR,
    license=LICENSE,
    author_email=AUTHOR_EMAIL,
    url=URL,
    install_requires=INSTALL_REQUIRES,
    packages=['thundera', 'thundera.libs'],
    classifiers=[
        'Programming Language :: Python :: 3',
        'Operating System :: POSIX :: Linux',
        'Development Status :: 3 - Alpha',
        'License :: OSI Approved :: Apache Software License'
    ],
    entry_points='''
        [console_scripts]
        thundera=thundera.thundera:cli
    ''',
)



import os
import time
import mimetypes
import hashlib
import tarfile
import gzip
import csv
import sys
from slugify import slugify
from zipfile import ZipFile
from tabulate import tabulate
from pathlib import Path
from . import File
from . import ArchiveHandler
from . import ErrorHandler


class FolderScanner:

    def __init__(self, rootfolder):

        self.debug = ErrorHandler.ErrorHandler(__name__)
        ts = time.time()

        # Files found during scanner
        self.filelist = []

        # Files to Exclude from ArchiveHandler
        self.exfiles = {}

        # List of folders
        self.folders = {}
        self.symbols = {}

        self.rootfolder = rootfolder
        self.scanFolder(rootfolder, '-')
        self.rfname = slugify(rootfolder)

        self.fresults = []
        self.aresults = []
        self.bsaqueue = []

        self.file_count
        self.lcounter = 0

        for fO in self.filelist:
            # print (fO.filename)
            row = [
                fO.checksum,
                fO.filename,
                fO.filetype,
                fO.isbinary,
                fO.isarchive,
                fO.forceign,
                fO.plang,
                fO.pfile,
                fO.filepath]
            if fO.isbinary and not fO.isarchive and not fO.forceign:
                self.bsaqueue.append(row)
            elif fO.isarchive and not fO.forceign:
                self.aresults.append(row)
            else:
                if not fO.forceign:
                    self.fresults.append(row)

        headers = [
            "Checksum",
            "Filename",
            "MimeType",
            "Binary",
            "Archive",
            "Ignore",
            "Language",
            "Parent",
            "Filepath"]

        with open(self.rfname+'-archive.csv', 'w', newline='') as file:
            writer = csv.writer(file)
            writer.writerow(headers)
            for row in self.aresults:
                writer.writerow(row)

        with open(self.rfname+'-files.csv', 'w', newline='') as file:
            writer = csv.writer(file)
            writer.writerow(headers)
            for row in self.fresults:
                writer.writerow(row)

        headers = [
            "Checksum",
            "Symbols"]

        rowsym = []
        for key in self.symbols:
            rowsym.append([key, self.symbols[key]])

        with open(self.rfname+'-symbols.csv', 'w', newline='') as file:
            writer = csv.writer(file)
            writer.writerow(headers)
            for row in rowsym:
                writer.writerow(row)

        # print(self.symbols)
        # for key in self.symbols:
        #    print(key, '->', self.symbols[key])

        dt = str(round(time.time() - ts, 2))
        self.debug.info('Scan took ' + dt + ' seconds')

    def progress(self, count, total):
        bar_len = 60
        filled_len = int(round(bar_len * count / float(total)))

        percents = round(100.0 * count / float(total), 1)
        bar = '=' * filled_len + '-' * (bar_len - filled_len)

        sys.stdout.write(
            '[%s] %s%s %s\r' % (bar, percents, '%', 'scanning...'))
        sys.stdout.flush()

    def scanFolder(self, rootfolder, pfile):
        if len(pfile.strip()) <= 1:
            eMSG = "* Preparing files to be scanned, please wait..."
            print(eMSG)
            kount = [len(files) for _, _, files in os.walk(rootfolder)]
            self.file_count = kount
            self.file_count = sum(self.file_count)
            print(' ', '>', self.file_count, 'files added to the queue.')
            self.lcounter = 0

        for (dirpath, dirnames, filenames) in os.walk(rootfolder):
            for f in filenames:
                if not Path(os.path.join(dirpath, f)).is_symlink():
                    if len(pfile.strip()) <= 1:
                        self.lcounter += 1
                        self.progress(self.lcounter, self.file_count)
                    fileInfo = File.File(self.debug, dirpath, f, pfile)
                    self.filelist.append(fileInfo)
                    symbols = fileInfo.get_symbols()
                    if len(symbols) >= 1:
                        tmpCSV = symbols[list(symbols.keys())[0]].split(',', 1)
                        self.symbols[tmpCSV[0]] = tmpCSV[1]

            for d in dirnames:
                self.folders[d] = dirpath
        if len(pfile.strip()) <= 1:
            sys.stdout.flush()
            print('')
        for ar in self.filelist:
            if ar.isarchive:
                print(' ', '>', ar.filename)
                archfile = os.path.join(ar.filepath, ar.filename)
                if archfile not in self.exfiles.keys():
                    target = os.path.join(ar.filepath, ar.slugify)
                    self.exfiles[archfile] = target
                    ArchiveHandler.ArchiveHandler(
                        self.debug,
                        ar.filepath,
                        ar.filename,
                        ar.filetype,
                        ar.slugify,
                        ar.checksum
                    )
                    self.scanFolder(target, ar.checksum)
